@@ -31,7 +31,12 @@
  * The important one here is not to allow ';' because we use that
  * internally as a list delimiter. See GCONF_DATABASE_LIST_DELIM
  */
-static const char invalid_chars[] = " \t\r\n\"$&<>,+=#!()'|{}[]?~`;%\\";
+static const char invalid_chars[] = 
+#ifndef G_OS_WIN32
+  /* Space is common in user names (and thus home directories) on Windows */
+  " "
+#endif
+  "\t\r\n\"$&<>,+=#!()'|{}[]?~`;%\\";
 
 static gboolean
 gconf_address_valid (const char  *address,
@@ -180,7 +185,7 @@ gconf_backend_file(const gchar* address)
 
   g_free(back);
 
-  if (gconf_file_exists(retval))
+  if (g_file_test(retval, G_FILE_TEST_EXISTS))
     {
       g_free(file);
 
@@ -200,7 +205,7 @@ gconf_backend_file(const gchar* address)
 
       g_free(dir);
       
-      if (gconf_file_exists(retval))
+      if (g_file_test(retval, G_FILE_TEST_EXISTS))
         {
           g_free(file);
           return retval;
@@ -331,7 +336,7 @@ gconf_get_backend(const gchar* address, GError** err)
           if (!g_module_supported())
             g_error(_("GConf won't work without dynamic module support (gmodule)"));
               
-          module = g_module_open(file, 0);
+          module = g_module_open(file, G_MODULE_BIND_LAZY);
               
           g_free(file);
           
